@@ -1,40 +1,17 @@
 import asyncio
 import dataclasses
-import functools
 import logging
 from typing import Any, Callable, Dict, Tuple
 
-
-try:
-    import orjson as json
-
-    def dumps_decorator(func: Callable[..., bytes]) -> Callable[..., str]:
-        @functools.wraps(func)
-        def wrapper(obj: Any) -> str:
-            return func(obj).decode()
-
-        return wrapper
-
-    def loads_decorator(func: Callable[[bytes], Any]) -> Callable[[str], Any]:
-        @functools.wraps(func)
-        def wrapper(json_str: str) -> Any:
-            return func(json_str.encode())
-
-        return wrapper
-
-    json.dumps = dumps_decorator(json.dumps)  # type: ignore
-    json.loads = loads_decorator(json.loads)  # type: ignore
-
-except ImportError:
-    import json  # type: ignore
-
+import orjson
 from broadcaster import Broadcast
-from jsonschema import ValidationError, validate
+from jsondaora import DeserializationError, asdataclass, dataclass_asjson
 
 from .entities import Specification
 from .exceptions import (
     ChannelOperationNotFoundError,
     InvalidChannelError,
+    InvalidMessageError,
     OperationIdNotFoundError,
 )
 
@@ -50,13 +27,43 @@ class AsyncApi:
     republish_error_messages: bool = True
     logger: logging.Logger = logging.getLogger(__name__)
 
-    async def publish(self, channel_id: str, message: Dict[str, Any]) -> None:
-        self.validate_message(channel_id, message)
+    async def connect(self) -> None:
         await self.broadcast.connect()
+
+    def payload(self, channel_id: str, **message: Any) -> Any:
+        type_ = self.payload_type(channel_id)
+
+        if type_:
+            return asdataclass(message, type_)
+
+        return message
+
+    async def publish_json(
+        self, channel_id: str, message: Dict[str, Any]
+    ) -> None:
         await self.broadcast.publish(
-            channel=channel_id, message=json.dumps(message)  # type: ignore
+            channel=channel_id,
+            message=self.parse_message(
+                channel_id, self.payload(channel_id, **message)
+            ).decode(),
         )
 
+    async def publish(self, channel_id: str, message: Any) -> None:
+        await self.broadcast.publish(
+            channel=channel_id,
+            message=self.parse_message(channel_id, message).decode(),
+        )
+
+    async def listen_all(self) -> None:
+        tasks = []
+
+        for channel_id in self.spec.channels.keys():
+            task = asyncio.create_task(self.listen(channel_id))
+            task.add_done_callback(task_callback)
+            tasks.append(task)
+
+        await asyncio.gather(*tasks)
+
     async def listen(self, channel_id: str) -> None:
         try:
             operation_id = self.spec.channels[
@@ -71,20 +78,15 @@ class AsyncApi:
         async with self.broadcast.subscribe(channel=channel_id) as subscriber:
             async for event in subscriber:
                 try:
-                    json_message = json.loads(event.message)
-                    self.validate_message(channel_id, json_message)
+                    json_message = orjson.loads(event.message)
+                    payload = self.payload(channel_id, **json_message)
 
-                    coro = self.operations[(channel_id, operation_id)](
-                        json_message
-                    )
+                    coro = self.operations[(channel_id, operation_id)](payload)
 
                     if asyncio.iscoroutine(coro):
                         await coro
 
-                except ValidationError:
-                    raise
-
-                except json.JSONDecodeError:
+                except (orjson.JSONDecodeError, DeserializationError):
                     raise
 
                 except KeyError:
@@ -98,14 +100,29 @@ class AsyncApi:
                         f'error={type(error).__name__}; '
                         f"message={event.message[:100]}"
                     )
-                    await self.publish(channel_id, json_message)
+                    await self.publish(channel_id, payload)
 
-    def validate_message(
-        self, channel_id: str, message: Dict[str, Any]
-    ) -> None:
+    def parse_message(self, channel_id: str, message: Any) -> Any:
         try:
-            schema = self.spec.channels[channel_id].subscribe.message.payload
+            type_ = self.spec.channels[channel_id].subscribe.message.payload
         except KeyError:
             raise InvalidChannelError(channel_id)
 
-        validate(instance=message, schema=schema)
+        if type_:
+            if not isinstance(message, type_):
+                raise InvalidMessageError(message, type_)
+
+            v = dataclass_asjson(message)
+            return v
+
+        return message
+
+    def payload_type(self, channel_id: str) -> Any:
+        try:
+            return self.spec.channels[channel_id].subscribe.message.payload
+        except KeyError:
+            raise InvalidChannelError(channel_id)
+
+
+def task_callback(future: Any) -> None:
+    future.result()
